package com.ibm.iru.db2express9_1fixpacklnx;

/**
 * @(#)DB2Express9_1FixpackLnxPDC.java
 *
 * Licensed Materials - Property of IBM
 *
 * 5724-J10
 *
 * (C) Copyright IBM Corp. 2007
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * This is sample code made available for use in accordance with terms set
 * forth in the license agreement document for the IBM Express Runtime.
 */

import com.ibm.iru.message.NLSKeys;
import com.ibm.jsdt.support.SupportLinuxBase;
import com.ibm.jsdt.support.SupportLinuxHelper;
import com.ibm.jsdt.support.installedproduct.DB2InstalledProduct;

/**
 * This class checks that DB2 Express Fixpack can be installed
 * under the specified installation directory. 
 *
 */
public class DB2Express9_1FixpackLnxPDC extends SupportLinuxBase
{
  private static final String copyright0 = "Licensed Materials - Property of IBM";
  private static final String copyright1 = "5724-J10";
  private static final String copyright2 = "(C) Copyright IBM Corp. 2007 All Rights Reserved.";
  private static final String copyright3 = "US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";

  private SupportLinuxHelper ivHelper;
  private String ivVersion = null;
  private String ivInstallDir = null;
  private DB2InstalledProduct ivDB2Product;

  /**
   * This method will initialize any necessary variables
   * @return SUCCESS if successful otherwise return FAILURE
   */
  private int init(String args[])
  {
    int rc = SUCCESS;

    setMainResources("com.ibm.iru.message.MessagesNLS");
    int no_Of_Expected_args = 0;
    ivHelper = getLinuxHelper();

    //if an incorrect number of arguments were passed log the error.
    if (no_Of_Expected_args != args.length)
    {
      String[] replacementText = { "DB2Express9_1FixpackLnxPDC", String.valueOf(no_Of_Expected_args),
          String.valueOf(args.length) };
      setMessage(getResourceString(NLSKeys.BAD_NUMBER_PGM_ARGS, replacementText));
      ivHelper.log(this);
      rc = FAILURE;
    }
    else
    {
      //version should be specified in three digits, i.e: 9.1.1
      ivVersion = ivHelper.getProductInstallingVersion(this);
    }

    return rc;
  }

  /**
   * Determine install action:
   * 
   * @return PDC_DOES_NOT_EXIST - If specified install directory contains DB2 base with a lower level fixpack
   *         PDC_EXIST -          If DB2 base with fixpack version is already installed
   *         FAILURE -            Wrong version of DB2 base is installed in the destination directory, or
   *                              specified directory is empty or contains something else, 
   *                              or base location is not specified  
   */
  private int determineInstallAction()
  {
    int returnCode = PDC_DOES_NOT_EXIST;
    String versionInstalled = null;
    ivInstallDir = ivHelper.getIbmNsiPropValue("installDir");

    //if no value then return FAILURE as installation location is a required parameter
    if ((ivInstallDir == null) || ivInstallDir.equals(""))
    {
      setMessage(getResourceString(NLSKeys.INSTALLATION_LOCATION_NOT_FOUND));
      ivHelper.postErrorMessageToDeployerUI(this);
      returnCode = FAILURE;
    }
    else
    {
      //check if DB2 is installed 
      ivDB2Product = DB2InstalledProduct.getInstalledProduct(this, ivInstallDir);
      if (ivDB2Product == null)
      {
        setMessage("Specified directory does not contain the DB2 Express installation. Specify the directory that contains DB2 Express version "
            + ivVersion.substring(0, 3));
        ivHelper.postErrorMessageToDeployerUI(this);
        returnCode = FAILURE;
      }
      else
      {
        //Get version installed up to fixpack level. Ex:9.1.0.1. 
        //DB2 Linux fixpack level modification is at the 4th digit.
        versionInstalled = ivDB2Product.getInstalledVersion().trim();

        //Make sure the first three digits are the same (base level). Should only be comparing versions that differ only in the
        //modification level, i.e 9.1.0.x levels.
        if (ivVersion.substring(0, 5).equals(versionInstalled.substring(0, 5)))
        {
          setVersion(ivVersion);
          setCompareVersion(versionInstalled);
          int compareResult = ivHelper.compareVersions(this);

          if (compareResult == 0)
          {
            //same version of DB2 fixpack is already installed
            returnCode = PDC_EXISTS;
          }
          else if (compareResult == 1)
          {
            //installed DB2 has a lower fixpack level, continue with installation.
            returnCode = PDC_DOES_NOT_EXIST;
          }
          else if (compareResult == -1)
          {
            //installed DB2 has a higher fixpack level than the one we are installing, stop installation.
            setMessage(getResourceString(NLSKeys.LATER_DB2_VER_INSTALLED, versionInstalled));
            ivHelper.postErrorMessageToDeployerUI(this);
            returnCode = FAILURE;
          }
          else
          {
            //error comparing the versions, version format is incorrect.
            setMessage("An error occurred while checking for the version of DB2 Express installed on the computer. "
                + versionInstalled + " has an unrecognized format.");
            ivHelper.postErrorMessageToDeployerUI(this);
            returnCode = FAILURE;
          }
        }
      }
    }

    return returnCode;
  }

  /**
   * Stop all DB2 processes and DAS. These are the steps listed as required in the readme file
   * for the DB2 fixpack. 
   * <li>1. Stop all DB2 processes
   * <li>2. Stop the DB2 Administration Server
   * <li>3. Ensure DB2 interprocess communications are cleaned for the instance to be updated
   
   * @return SUCCESS if the system is ready for fixpack install, FAILURE otherwise.
   */
  private int stopDB2()
  {
    int rc = SUCCESS;
    ivDB2Product = DB2InstalledProduct.getInstalledProduct(this, ivInstallDir);
    rc = ivDB2Product.stopDB2(ivInstallDir);
    if (rc == SUCCESS)
    {
      rc = ivDB2Product.stopDAS();
    }
    if (rc == SUCCESS)
    {
      rc = ivDB2Product.ipClean(ivInstallDir);
    }
    return rc;
  }

  /**
   * Entry point
   */
  public static void main(String args[])
  {
    // Create and initialize the object
    DB2Express9_1FixpackLnxPDC pdc = new DB2Express9_1FixpackLnxPDC();
    int rc = pdc.init(args);

    if (rc == SUCCESS)
    {
      // Check if DB2 fixpack is already installed 
      rc = pdc.determineInstallAction();
    }

    if (rc == PDC_DOES_NOT_EXIST)
    {
      if (pdc.stopDB2() == FAILURE)
      {
        rc = FAILURE;
      }
    }
    System.exit(rc);
  }
}