package com.ibm.iru.db2express9_1fixpackwin;

/**
 * @(#)DB2Express9_1FixpackWinPDC.java
 *
 * Licensed Materials - Property of IBM
 *
 * 5724-J10
 *
 * (C) Copyright IBM Corp. 2007
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * This is sample code made available for use in accordance with terms set
 * forth in the license agreement document for the IBM Express Runtime.
 */

import com.ibm.iru.message.NLSKeys;
import com.ibm.jsdt.support.SupportWindowsBase;
import com.ibm.jsdt.support.SupportWindowsHelper;
import com.ibm.jsdt.support.installedproduct.DB2InstalledProduct;

/**
 * This class checks if DB2 Express Fixpack is installed on the system
 * 
 * */
public class DB2Express9_1FixpackWinPDC extends SupportWindowsBase
{
  private static final String copyright0 = "Licensed Materials - Property of IBM";
  private static final String copyright1 = "5724-J10";
  private static final String copyright2 = "(C) Copyright IBM Corp. 2007 All Rights Reserved.";
  private static final String copyright3 = "US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";

  private String ivVersion = null;
  private String ivInstallDir = null;
  private SupportWindowsHelper ivHelper;
  private DB2InstalledProduct ivDB2Product;

  /**
   * Initializes the framework and validates arguments
   * Expects 1 arguments: 
   * @param arg[0] = response file name
   * @return SUCCESS = Initialization successful; 
   *         FAILURE = Arguments incorrect or other initialization failed 
   */
  private int init(String[] args)
  {
    int rc = SUCCESS;
    int no_Of_Expected_args = 1;
    setMainResources("com.ibm.iru.message.MessagesNLS");
    ivHelper = getWindowsHelper();

    // Ensure the right number of parameters was passed
    if (no_Of_Expected_args == args.length)
    {
      setResponseFileName(args[0].trim());
      ivVersion = ivHelper.getProductInstallingVersion(this);
    }
    else
    {
      String[] replacementText = { "DB2Express9_1FixpackWinPDC", String.valueOf(no_Of_Expected_args),
          String.valueOf(args.length) };
      ivHelper.log(getResourceString(NLSKeys.BAD_NUMBER_PGM_ARGS, replacementText));
      rc = FAILURE;
    }

    return rc;
  }

  /**
   * determine install action:
   * 
   * @return PDC_DOES_NOT_EXIST - If specified install directory contains DB2 base with a lower level fixpack
   *         PDC_EXIST -          If DB2 base with fixpack version is already installed
   *         FAILURE -            Wrong version of DB2 base is installed in the destination directory, or
   *                              specified directory is empty or contains something else, 
   *                              or base location and/or DB2 copy is not specified  
   */
  private int determineInstallAction()
  {
    int returnCode = PDC_DOES_NOT_EXIST;
    String versionInstalled = null;
    setKey("FILE");
    ivInstallDir = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));

    //if no value then return FAILURE as installation location is a required parameter
    if ((ivInstallDir == null) || ivInstallDir.equals(""))
    {
      setMessage(getResourceString(NLSKeys.INSTALLATION_LOCATION_NOT_FOUND));
      ivHelper.postErrorMessageToDeployerUI(this);
      returnCode = FAILURE;
    }
    else
    {
      //check if DB2 is installed 
      ivDB2Product = DB2InstalledProduct.getInstalledProduct(this, ivInstallDir);
      if (ivDB2Product == null)
      {
        setMessage("Specified directory does not contain the DB2 Express installation. Specify the directory that contains DB2 Express version"
            + ivVersion.substring(0,3));
        ivHelper.postErrorMessageToDeployerUI(this);
        returnCode = FAILURE;
      }
      else
      {
        //get version installed up to fixpack level. Ex:9.1.0
        //DB2 Windows fixpack level modification is at 3rd digit
        versionInstalled = ivDB2Product.trimVersionToNVersionPlaces(3, ivDB2Product.getInstalledVersion().trim());

        //Make sure the first two digits are the same (base level). Should only be comparing versions that differ only in the
        //modification level, i.e 9.1.x levels.
        if (ivVersion.substring(0, 3).equals(versionInstalled.substring(0, 3)))
        {
          setVersion(ivVersion);
          setCompareVersion(versionInstalled);
          int compareResult = ivHelper.compareVersions(this);

          if (compareResult == 0)
          {
            //same version of DB2 fixpack is already installed
            returnCode = PDC_EXISTS;
          }
          else if (compareResult == 1)
          {
            //installed DB2 has a lower fixpack level, continue with installation.
            returnCode = PDC_DOES_NOT_EXIST;
          }
          else if (compareResult == -1)
          {
            //installed DB2 has a higher fixpack level than the one we are installing, stop installation.
            setMessage(getResourceString(NLSKeys.LATER_DB2_VER_INSTALLED, versionInstalled));
            ivHelper.postErrorMessageToDeployerUI(this);
            returnCode = FAILURE;
          }
          else
          {
            //error comparing the versions, version format is incorrect.
            setMessage("An error occurred while checking for the version of DB2 Express installed on the computer. Version "
                + versionInstalled + " has an unrecognized format.");
            ivHelper.postErrorMessageToDeployerUI(this);
            returnCode = FAILURE;
          }
        }
      }
    }

    return returnCode;
  }

  /**
   * Stops DB2 processes. DB2 processes need to be stopped before fixpack upgrade.
   * @ return SUCCESS, FAILURE, it may also return warnings i.e if the processes are already stopped
   */
  private int stopDB2()
  {
    return ivDB2Product.stopDB2(ivInstallDir);
  }

  /**
   * Expects the following parameter to be passed in:
   * parm#0 = response file name
   */
  public static void main(String args[])
  {
    // Create and initialize the object
    DB2Express9_1FixpackWinPDC pdc = new DB2Express9_1FixpackWinPDC();
    int retVal = pdc.init(args);

    // Check if DB2 Fixpack is already installed and validate variables
    if (retVal == SUCCESS)
    {
      retVal = pdc.determineInstallAction();
    }
    if (retVal == PDC_DOES_NOT_EXIST)
    {
      if (pdc.stopDB2() == FAILURE)
      {
        retVal = FAILURE;
      }
    }
    System.exit(retVal);
  }
}
